<?php 
	class PerelinkPluginSource {
		const SERVER_URL = 'https://perelink.pro';

		private static $data = [];

		/**
		 * @var PerelinkPluginOptions
		 */
		private $options;

		/**
		 * @var CurlHelper
		 */
		private $curl;

		private $force_url;

		public function __construct($options, $force_url = false) {
			$this->options = $options;

			if ($force_url) {
				$this->force_url = $force_url;
			}

			$this->initCurl();
		}

		private function initCurl() {
			$this->curl                 = new CurlHelper();
			$this->curl->header         = 0;
			$this->curl->timeout        = 3;
			$this->curl->ssl_verifypeer = false;
			$this->curl->returnTransfer = true;
		}

		private function preapareUrl( UrlHelper $url ) {
			if ( trim( $this->options['protectionCode'] ) ) {
				$url->params['code'] = trim( $this->options['protectionCode'] );
			}
		}

		private function createUrl() {
			$url = new UrlHelper( self::SERVER_URL );
			$this->preapareUrl( $url );
			return $url;
		}

		private function getCurrentUrl() {
			if ($this->force_url) {
				return $this->force_url;	
			} else {
				return 'http://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
				// return 'https://nastroikezhenshinenemesto.ru/pol/uteplenie-pola-v-derevyannom-dome.html';
			}
		}

		public function getDataByPage( ) {
			$pageUrl = new UrlHelper( $this->getCurrentUrl() );

			if ( ! array_key_exists( $pageUrl->toString(), self::$data ) ) {
				$url                 = $this->createUrl();
				$url->path           = '/service/otvet.php';
				$url->params['proj'] = ($pageUrl->sheme == 'https' ? 'https://' : '') . $pageUrl->host;
				$url->params['url']  = $pageUrl->toString();

				$this->curl->url     = $url->toString();
				$this->curl->execute();

				$data  = $this->curl->getBody();
				$array = json_decode( $data );
				if ( ! is_array( $array ) ) {
					$array = [];
				}
				self::$data[$pageUrl->toString()] = $array;
			}

			return self::$data[$pageUrl->toString()];
		}

		public function getAfterText( $count = 0 ) {
			$afterText = [];
			$data      = $this->getDataByPage( );
			if ( ! is_array( $data ) ) {
				return $afterText;
			}
			foreach ( $data as $item ) {
				if ( $this->isAfterText( $item ) ) {
					$afterText[] = $this->map( $item );
				}
			}

			if ( $count ) {
				return array_slice( $afterText, 0, $count );
			} else {
				return $afterText;
			}
		}

		private function isAfterText( $item ) {
			return $item[3] == 'posle';
		}

		public function getInContent( ) {
			$inContent = [];
			$data      = $this->getDataByPage( );
			if ( ! is_array( $data ) ) {
				return $inContent;
			}
			foreach ( $data as $item ) {
				if ( $this->isInContent( $item ) ) {
					$inContent[] = $this->map( $item );
				}
			}
			return $inContent;
		}

		private function isInContent( $item ) {
			return $item[3] == 'text';
		}

		private function map( $item ) {
			$res              = new PerelinkPluginRelation;
			$res->url         = $item[1];
			$res->anchor      = $item[4];
			$res->environment = $item[5];
			$res->keywords    = $item[2];
			return $res;
		}
	}
?>